"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var global_state_1 = require("../global-state");

var native = WeakMap.prototype;
var getters = ["has", "get"];
var all = ["set", "delete"].concat(getters);

function shim(target, bindCurrentReaction, queueRunReactions, proxyValue) {
  target.$raw = {};
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    var _loop = function _loop() {
      var method = _step.value;

      // tslint:disable-next-line:space-before-function-paren only-arrow-functions
      target.$raw[method] = function () {
        native[method].apply(target, arguments);
      };
    };

    for (var _iterator = all[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      _loop();
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  var _loop2 = function _loop2() {
    var getter = getters[_i];

    // tslint:disable-next-line:space-before-function-paren only-arrow-functions
    target[getter] = function (key) {
      var value = native[getter].apply(this, arguments);
      global_state_1.globalState.event.emit("get", {
        target: target,
        key: null,
        value: value
      });
      value = proxyValue(this, key, value);
      bindCurrentReaction(this, key);
      return value;
    };
  };

  for (var _i = 0; _i < getters.length; _i++) {
    _loop2();
  } // tslint:disable-next-line:space-before-function-paren only-arrow-functions


  target.set = function (key, value) {
    var oldValue = this.get(key);
    var result = native.set.apply(this, arguments);
    global_state_1.globalState.event.emit("set", {
      target: target,
      key: key,
      oldValue: oldValue,
      value: value
    });

    if (oldValue !== value) {
      queueRunReactions(this, key);
    }

    return result;
  }; // tslint:disable-next-line:space-before-function-paren only-arrow-functions


  target.delete = function (key) {
    var has = this.has(key);
    var result = native.delete.apply(this, arguments);
    global_state_1.globalState.event.emit("deleteProperty", {
      target: target,
      key: key
    });

    if (has) {
      queueRunReactions(this, key);
    }

    return result;
  };

  return target;
}

exports.default = shim;