"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

Object.defineProperty(exports, "__esModule", {
  value: true
});

var global_state_1 = require("./global-state");

var utils_1 = require("./utils");

var Reaction =
/*#__PURE__*/
function () {
  function Reaction(name, callback, delay) {
    (0, _classCallCheck2.default)(this, Reaction);
    // Run reaction's delay.
    this.delay = null; // Binders for this reaction.
    // Binder include callback, and it's detail info, like delay.

    this.keyBinders = new Set();
    this.name = name;
    this.callback = callback;
    this.delay = delay || null;
  }
  /**
   * The variables accessed in track are bound to the current reaction, which triggers binder's callback when the variables are modified.
   * @TODO: Current only support synchronization, do not support asynchronous!
   */


  (0, _createClass2.default)(Reaction, [{
    key: "track",
    value: function track(callback) {
      if (!callback) {
        return;
      } // If it is already in track, add this directly to the pendingTrack queue, and return.
      // When the lastest runObserver is executed, current pendingTrack will executed.


      if (utils_1.inTrack()) {
        global_state_1.globalState.pendingTracks.add(this.track.bind(this, callback));
        return;
      }
      /**
       * Run callback.
       */


      global_state_1.globalState.currentReaction = this; // Clear bindings first.

      this.clearBinding();

      try {
        callback({
          debugId: global_state_1.globalState.currentDebugId
        });
      } finally {
        global_state_1.globalState.currentReaction = null;
      }
      /**
       * Execute the rest of the track.
       */


      var currentRunCount = 0;
      var MAX_RUN_COUNT = 1000; // Copy pendingTracksqueue to prevent dead loop.
      // When the copy is finished, empty the origin queue, so that the execution is added to the clean new queue.

      var pendingTacksCopy = Array.from(global_state_1.globalState.pendingTracks);
      global_state_1.globalState.pendingTracks.clear();
      pendingTacksCopy.forEach(function (eachTack) {
        currentRunCount++;

        if (currentRunCount >= MAX_RUN_COUNT) {
          // tslint:disable-next-line:no-console
          console.warn("The number of executions reaches the upper limit, there may be a dead cycle.");
          global_state_1.globalState.pendingTracks.clear();
          return;
        }

        eachTack();
      });
    }
    /**
     * Destroy
     */

  }, {
    key: "dispose",
    value: function dispose() {
      this.clearBinding();
      this.callback = null;
    }
    /**
     * Run reaction's callback.
     */

  }, {
    key: "run",
    value: function run() {
      if (this.callback) {
        this.callback();
      }
    }
    /**
     * Add binder.
     */

  }, {
    key: "addBinder",
    value: function addBinder(keyBinder) {
      this.keyBinders.add(keyBinder);
    }
    /**
     * Clear bindings.
     */

  }, {
    key: "clearBinding",
    value: function clearBinding() {
      var _this = this;

      // Clear all binder which reference this.
      this.keyBinders.forEach(function (keyBinder) {
        keyBinder.delete(_this);
      }); // Clear self keyBinders.

      this.keyBinders.clear();
    }
  }]);
  return Reaction;
}();

exports.Reaction = Reaction;