"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var __decorate = void 0 && (void 0).__decorate || function (decorators, target, key, desc) {
  var c = arguments.length,
      r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc,
      d;
  if ((typeof Reflect === "undefined" ? "undefined" : (0, _typeof2.default)(Reflect)) === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);else for (var i = decorators.length - 1; i >= 0; i--) {
    if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  }
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

var index_1 = require("../src/index"); // /**
//  * observable
//  */


test('should return a new observable when no argument is provided', function () {
  var dynamicObj = index_1.observable();
  expect(index_1.isObservable(dynamicObj)).toBe(true);
});
test('should return an observable wrapping of an object argument', function () {
  var obj = {
    prop: 'value'
  };
  var dynamicObj = index_1.observable(obj);
  expect(obj === dynamicObj).not.toBe(true);
  expect(index_1.isObservable(dynamicObj)).toBe(true);
});
test('should return the argument if test is already an observable', function () {
  var dynamicObj1 = index_1.observable();
  var dynamicObj2 = index_1.observable(dynamicObj1);
  expect(dynamicObj1 === dynamicObj2).toBe(true);
});
test('should return the same observable wrapper when called repeatedly with the same argument', function () {
  var obj = {
    prop: 'value'
  };
  var dynamicObj1 = index_1.observable(obj);
  var dynamicObj2 = index_1.observable(obj);
  expect(dynamicObj1 === dynamicObj2).toBe(true);
});
test('should never modify the underlying plain object', function () {
  var obj = {};
  var dynamicObj = index_1.observable(obj);
  obj.nested1 = {};
  dynamicObj.nested2 = index_1.observable({});
  expect(index_1.isObservable(obj.nested1)).not.toBe(true);
  expect(index_1.isObservable(obj.nested2)).not.toBe(true);
});
test('null throw typeError', function () {
  expect(function () {
    index_1.observable(null);
  }).toThrow(TypeError);
});
test('undefined not throw', function () {
  expect(function () {
    index_1.observable(undefined);
  }).not.toThrow();
});
test('primitive type typeError', function () {
  expect(function () {
    index_1.observable(1);
  }).toThrow(TypeError);
});
test('when observable class has constructor', function () {
  var MyCount =
  /*#__PURE__*/
  function () {
    function MyCount(count1, count2) {
      (0, _classCallCheck2.default)(this, MyCount);
      this.count1 = count1;
      this.count2 = count2;
    }

    (0, _createClass2.default)(MyCount, [{
      key: "getCount1",
      value: function getCount1() {
        return this.count1;
      }
    }, {
      key: "getCount2",
      value: function getCount2() {
        return this.count2;
      }
    }]);
    return MyCount;
  }();

  MyCount = __decorate([index_1.observable], MyCount);
  var myCount = new MyCount(3, 4);
  expect(myCount.getCount1() === 3).toBe(true);
  expect(myCount.getCount2() === 4).toBe(true);
});
/**
 * isObservable
 */

test('should throw a TypeError on invalid arguments', function () {
  expect(index_1.isObservable({})).not.toBe(true);
});
test('should return true if an observable is passed as argument', function () {
  var dynamicObj = index_1.observable();
  expect(index_1.isObservable(dynamicObj)).toBe(true);
});
test('should return false if a non observable is passed as argument', function () {
  var obj1 = {
    prop: 'value'
  };
  var obj2 = new Proxy({}, {});
  expect(index_1.isObservable(obj1)).not.toBe(true);
  expect(index_1.isObservable(obj2)).not.toBe(true);
});
/**
 * Action
 */

test('observe run sync', function () {
  var data = '';
  var dynamicObj = index_1.observable({
    name: 'b'
  });
  data += 'a';
  index_1.observe(function () {
    return data += dynamicObj.name;
  });
  data += 'c';
  return Promise.resolve().then(function () {
    return expect(data === 'abc').toBe(true);
  });
});
test('observe run but not used', function () {
  var runCount = 0;
  var dynamicObj = index_1.observable({
    name: 'b'
  });
  index_1.observe(function () {
    return runCount++;
  });
  dynamicObj.name = 'c';
  return Promise.resolve().then(function () {
    return expect(runCount === 1).toBe(true);
  });
});
test('observe run any time', function () {
  var runCount = 0;
  var dynamicObj = index_1.observable({
    name: 'b'
  });
  index_1.observe(function () {
    runCount += 1; // use it!
    // tslint:disable-next-line:no-unused-expression

    dynamicObj.name;
  });
  dynamicObj.name = 'c';
  dynamicObj.name = 'd';
  return Promise.resolve().then(function () {
    return expect(runCount === 3).toBe(true);
  });
});
test('Action', function () {
  var runCount = 0;
  var count = 0;
  var dynamicObj = index_1.observable({
    number: 1
  });
  index_1.observe(function () {
    runCount += 1; // use it!

    count += dynamicObj.number;
  });
  index_1.Action(function () {
    dynamicObj.number = 2;
    dynamicObj.number = 3;
  });
  return Promise.resolve().then(function () {
    return expect(runCount === 2).toBe(true);
  }).then(function () {
    return expect(count === 4).toBe(true);
  }); // 1 + 3, do not use 2
});
/**
 * Action
 */

test('Action with Action work', function () {
  var runCount = 0;
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  index_1.observe(function () {
    runCount++;
    data += dynamicObj.counter;
  });

  var MyAction =
  /*#__PURE__*/
  function () {
    function MyAction() {
      (0, _classCallCheck2.default)(this, MyAction);
    }

    (0, _createClass2.default)(MyAction, [{
      key: "run",
      value: function run() {
        dynamicObj.counter = 1;
        dynamicObj.counter = 2;
      }
    }]);
    return MyAction;
  }();

  __decorate([index_1.Action], MyAction.prototype, "run", null);

  new MyAction().run();
  return Promise.resolve().then(function () {
    return expect(runCount === 2).toBe(true);
  }).then(function () {
    return data = 2;
  }); // 0 + 2, ignore 1
});
test('Action with params', function () {
  var runCount = 0;
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  index_1.observe(function () {
    runCount++;
    data += dynamicObj.counter;
  });

  var MyAction =
  /*#__PURE__*/
  function () {
    function MyAction() {
      (0, _classCallCheck2.default)(this, MyAction);
    }

    (0, _createClass2.default)(MyAction, [{
      key: "run",
      value: function run(num) {
        dynamicObj.counter = num + 1;
        dynamicObj.counter = num + 1;
      }
    }]);
    return MyAction;
  }();

  __decorate([index_1.Action], MyAction.prototype, "run", null);

  new MyAction().run(1);
  return Promise.resolve().then(function () {
    return expect(runCount === 2).toBe(true);
  }).then(function () {
    return data = 3;
  }); // 0 + 3, ignore 2
});
test('Action can get sync return value', function () {
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  index_1.observe(function () {
    data += dynamicObj.counter;
  });

  var MyAction =
  /*#__PURE__*/
  function () {
    function MyAction() {
      (0, _classCallCheck2.default)(this, MyAction);
    }

    (0, _createClass2.default)(MyAction, [{
      key: "run",
      value: function run(num) {
        dynamicObj.counter = num + 1;
        return dynamicObj.counter;
      }
    }]);
    return MyAction;
  }();

  __decorate([index_1.Action], MyAction.prototype, "run", null);

  var result = new MyAction().run(1);
  return Promise.resolve().then(function () {
    return expect(result === 2).toBe(true);
  });
});
test('Action can get async return value',
/*#__PURE__*/
(0, _asyncToGenerator2.default)(
/*#__PURE__*/
_regenerator.default.mark(function _callee2() {
  var data, dynamicObj, MyAction, result;
  return _regenerator.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          data = 0;
          dynamicObj = index_1.observable({
            counter: 0
          });
          index_1.observe(function () {
            data += dynamicObj.counter;
          });

          MyAction =
          /*#__PURE__*/
          function () {
            function MyAction() {
              (0, _classCallCheck2.default)(this, MyAction);
            }

            (0, _createClass2.default)(MyAction, [{
              key: "run",
              value: function () {
                var _run = (0, _asyncToGenerator2.default)(
                /*#__PURE__*/
                _regenerator.default.mark(function _callee(num) {
                  return _regenerator.default.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          dynamicObj.counter = num + 1;
                          return _context.abrupt("return", dynamicObj.counter);

                        case 2:
                        case "end":
                          return _context.stop();
                      }
                    }
                  }, _callee, this);
                }));

                function run(_x) {
                  return _run.apply(this, arguments);
                }

                return run;
              }()
            }]);
            return MyAction;
          }();

          __decorate([index_1.Action], MyAction.prototype, "run", null);

          result = new MyAction().run(1);
          return _context2.abrupt("return", Promise.resolve().then(function () {
            return result.then(function (value) {
              expect(value === 2).toBe(true);
            });
          }));

        case 7:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2, this);
})));
/**
 * observe
 */

test('should observe basic properties', function () {
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  index_1.observe(function () {
    return data = dynamicObj.counter;
  });
  return Promise.resolve().then(function () {
    return expect(data === 0).toBe(true);
  }).then(function () {
    return dynamicObj.counter = 7;
  }).then(function () {
    return expect(data === 7).toBe(true);
  });
});
test('should observe delete operations', function () {
  var data = '';
  var dynamicObj = index_1.observable({
    prop: 'value'
  });
  index_1.observe(function () {
    return data = dynamicObj.prop;
  });
  return Promise.resolve().then(function () {
    return expect(data === 'value').toBe(true);
  }).then(function () {
    return delete dynamicObj.prop;
  }).then(function () {
    return expect(data === undefined).toBe(true);
  });
});
test('should observe properties on the prototype chain', function () {
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  var parentDynamicObj = index_1.observable({
    counter: 2
  });
  Object.setPrototypeOf(dynamicObj, parentDynamicObj);
  index_1.observe(function () {
    return data = dynamicObj.counter;
  });
  return Promise.resolve().then(function () {
    return expect(data === 0).toBe(true);
  }).then(function () {
    return delete dynamicObj.counter;
  }).then(function () {
    return expect(data === 2).toBe(true);
  }).then(function () {
    return parentDynamicObj.counter = 4;
  }).then(function () {
    return expect(data === 4).toBe(true);
  }).then(function () {
    return dynamicObj.counter = 3;
  }).then(function () {
    return expect(data === 3).toBe(true);
  });
});
test('should observe function call chains', function () {
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  index_1.observe(function () {
    return data = getCounter();
  });

  function getCounter() {
    return dynamicObj.counter;
  }

  return Promise.resolve().then(function () {
    return expect(data === 0).toBe(true);
  }).then(function () {
    return dynamicObj.counter = 2;
  }).then(function () {
    return expect(data === 2).toBe(true);
  });
});
test('should observe for of iteration', function () {
  var data = '';
  var dynamicObj = index_1.observable({
    array: ['Hello']
  });
  index_1.observe(function () {
    return data = dynamicObj.array.join(' ');
  });
  return Promise.resolve().then(function () {
    return expect(data === 'Hello').toBe(true);
  }).then(function () {
    return dynamicObj.array.push('World!');
  }).then(function () {
    return expect(data === 'Hello World!').toBe(true);
  }).then(function () {
    return dynamicObj.array.shift();
  }).then(function () {
    return expect(data === 'World!').toBe(true);
  });
}); // // test('should observe for in iteration', () => {
// //     let data = 0
// //     const dynamicObj: any = observable({prop: 0})
// //     observe(() => {
// //         data = 0
// //         for (let key in dynamicObj) {
// //             data += dynamicObj[key]
// //         }
// //     })
// //
// //     return Promise.resolve()
// //         .then(() => expect(data === 0))
// //         .then(() => dynamicObj.prop = 1)
// //         .then(() => expect(data === 1))
// //         .then(() => dynamicObj.prop1 = 1)
// //         .then(() => expect(data === 2))
// //         .then(() => dynamicObj.prop2 = 3)
// //         .then(() => expect(data === 5))
// //         .then(() => dynamicObj.prop1 = 6)
// //         .then(() => expect(data === 10))
// // })
// // test('should not observe well-known symbols', () => {
// //     let data = ''
// //     const dynamicObj = observable({[Symbol.toStringTag]: 'myString'})
// //     observe(() => data = String(dynamicObj))
// //
// //     return Promise.resolve()
// //         .then(() => expect(data).to.equal('[object myString]'))
// //         .then(() => dynamicObj[Symbol.toStringTag] = 'otherString')
// //         .then(() => expect(data).to.equal('[object myString]'))
// // })

test('should not observe set operations without a value change', function () {
  var data = '';
  var dynamicObj = index_1.observable({
    prop: 'prop'
  });
  var numOfRuns = 0;

  function testObserve() {
    data = dynamicObj.prop;
    numOfRuns++;
  }

  index_1.observe(testObserve);
  return Promise.resolve().then(function () {
    return expect(data === 'prop').toBe(true);
  }).then(function () {
    return dynamicObj.prop = 'prop';
  }).then(function () {
    expect(numOfRuns === 1).toBe(true);
    expect(data === 'prop').toBe(true);
  }).then(function () {
    return dynamicObj.prop = 'prop2';
  }).then(function () {
    return dynamicObj.prop = 'prop2';
  }).then(function () {
    expect(numOfRuns === 2).toBe(true);
    expect(data === 'prop2').toBe(true);
  });
});
test('should run synchronously after registration', function () {
  var data = '';
  var dynamicObj = index_1.observable({
    prop: 'prop'
  });
  var numOfRuns = 0;
  index_1.observe(function () {
    data = dynamicObj.prop;
    numOfRuns++;
  });
  expect(numOfRuns === 1).toBe(true);
  expect(data === 'prop').toBe(true);
  return Promise.resolve().then(function () {
    expect(numOfRuns === 1).toBe(true);
    expect(data === 'prop').toBe(true);
  }).then(function () {
    dynamicObj.prop = 'new prop';
  }).then(function () {
    expect(numOfRuns === 2).toBe(true);
    expect(data === 'new prop').toBe(true);
  });
});
test('should rerun maximum once per stack', function () {
  var data = 0;
  var dynamicObj = index_1.observable({
    prop1: 0,
    prop2: 0
  });
  var numOfRuns = 0;

  function testObserve() {
    data = dynamicObj.prop1 + dynamicObj.prop2;
    numOfRuns++;
  }

  index_1.observe(testObserve);
  return Promise.resolve().then(function () {
    expect(numOfRuns === 1).toBe(true);
    expect(data === 0).toBe(true);
  }).then(function () {
    index_1.Action(function () {
      dynamicObj.prop1 = 1;
      dynamicObj.prop2 = 3;
      dynamicObj.prop1 = 2;
    });
  }).then(function () {
    expect(numOfRuns === 2).toBe(true);
    expect(data === 5).toBe(true);
  });
});
test('should avoid infinite loops', function () {
  var dynamicObj1 = index_1.observable({
    prop: 'value1'
  });
  var dynamicObj2 = index_1.observable({
    prop: 'value2'
  });
  var numOfRuns1 = 0;
  var numOfRuns2 = 0;

  function test1() {
    dynamicObj1.prop = dynamicObj2.prop;
    numOfRuns1++;
  }

  function test2() {
    dynamicObj2.prop = dynamicObj1.prop;
    numOfRuns2++;
  }

  index_1.observe(test1);
  index_1.observe(test2);
  return Promise.resolve().then(function () {
    return dynamicObj1.prop = 'Hello';
  }).then(function () {
    return expect(dynamicObj2.prop === 'Hello').toBe(true);
  }).then(function () {
    return dynamicObj1.prop = 'World!';
  }).then(function () {
    return expect(dynamicObj2.prop === 'World!').toBe(true);
  }).then(function () {
    expect(numOfRuns1 === 3).toBe(true);
    expect(numOfRuns2 === 3).toBe(true);
  });
});
test('should return an unobserve (object) signal', function () {
  var data = 0;
  var dynamicObj = index_1.observable({
    counter: 0
  });
  var signal = index_1.observe(function () {
    return data = dynamicObj.counter;
  });
  expect((0, _typeof2.default)(signal) === 'object').toBe(true);
});
/**
 * set
 */

test('set should observe string', function () {
  var data;
  var dynamicObj = index_1.observable(new Set());
  index_1.observe(function () {
    return data = dynamicObj.has('value');
  });
  return Promise.resolve().then(function () {
    return expect(data).not.toBe(true);
  }).then(function () {
    return dynamicObj.add('value');
  }).then(function () {
    return expect(data).toBe(true);
  }).then(function () {
    return dynamicObj.delete('value');
  }).then(function () {
    return expect(data).not.toBe(true);
  });
});
test('should observe iteration', function () {
  var data;
  var dynamicObj = index_1.observable(new Set());
  index_1.observe(function () {
    data = 0;
    dynamicObj.forEach(function (each) {
      data += each;
    });
  });
  return Promise.resolve().then(function () {
    return expect(data === 0).toBe(true);
  }).then(function () {
    return dynamicObj.add(3);
  }).then(function () {
    return expect(data === 3).toBe(true);
  }).then(function () {
    return dynamicObj.add(2);
  }).then(function () {
    return expect(data === 5).toBe(true);
  }).then(function () {
    return dynamicObj.delete(2);
  }).then(function () {
    return expect(data === 3).toBe(true);
  }).then(function () {
    return dynamicObj.clear();
  }).then(function () {
    return expect(data === 0).toBe(true);
  });
});
test('set delete test', function () {
  var data;
  var numOfRuns = 0;
  var dynamicObj = index_1.observable(new Set());
  index_1.observe(function () {
    numOfRuns++;
    data = dynamicObj.has('value');
  });
  return Promise.resolve().then(function () {
    expect(data).not.toBe(true);
    expect(numOfRuns === 1).toBe(true);
  }).then(function () {
    return dynamicObj.add('value');
  }).then(function () {
    return dynamicObj.add('value');
  }).then(function () {
    expect(data).toBe(true);
    expect(numOfRuns === 2).toBe(true);
  }).then(function () {
    return dynamicObj.delete('value');
  }).then(function () {
    return dynamicObj.delete('value');
  }).then(function () {
    expect(data).not.toBe(true);
    expect(numOfRuns === 3).toBe(true);
  });
});
test('should observe mutations', function () {
  var data;
  var value = {};
  var dynamicObj = index_1.observable(new Set());
  index_1.observe(function () {
    return data = dynamicObj.has(value);
  });
  return Promise.resolve().then(function () {
    return expect(data).not.toBe(true);
  }).then(function () {
    return dynamicObj.add(value);
  }).then(function () {
    return expect(data).toBe(true);
  }).then(function () {
    return dynamicObj.delete(value);
  }).then(function () {
    return expect(data).not.toBe(true);
  });
});
test('should not observe non value changing mutations', function () {
  var data;
  var value = {};
  var numOfRuns = 0;
  var dynamicObj = index_1.observable(new Set());
  index_1.observe(function () {
    numOfRuns++;
    data = dynamicObj.has(value);
  });
  return Promise.resolve().then(function () {
    expect(data).not.toBe(true);
    expect(numOfRuns === 1).toBe(true);
  }).then(function () {
    return dynamicObj.add(value);
  }).then(function () {
    return dynamicObj.add(value);
  }).then(function () {
    expect(data).toBe(true);
    expect(numOfRuns === 2).toBe(true);
  }).then(function () {
    return dynamicObj.delete(value);
  }).then(function () {
    return dynamicObj.delete(value);
  }).then(function () {
    expect(data).not.toBe(true);
    expect(numOfRuns === 3).toBe(true);
  });
});
/**
 * WeakSet
 */

test('weakSet should observe object', function () {
  var data;
  var value = {};
  var dynamicObj = index_1.observable(new WeakSet());
  index_1.observe(function () {
    return data = dynamicObj.has(value);
  });
  return Promise.resolve().then(function () {
    return expect(data).not.toBe(true);
  }).then(function () {
    return dynamicObj.add(value);
  }).then(function () {
    return expect(data).toBe(true);
  }).then(function () {
    return dynamicObj.delete(value);
  }).then(function () {
    return expect(data).not.toBe(true);
  });
});
/**
 * Map
 */

test('weakSet should observe string', function () {
  var data;
  var dynamicObj = index_1.observable(new Map());
  index_1.observe(function () {
    return data = dynamicObj.get('key');
  });
  return Promise.resolve().then(function () {
    return expect(data === undefined).toBe(true);
  }).then(function () {
    return dynamicObj.set('key', 'value');
  }).then(function () {
    return expect(data === 'value').toBe(true);
  }).then(function () {
    return dynamicObj.delete('key');
  }).then(function () {
    return expect(data === undefined).toBe(true);
  });
});
test('should observe iteration', function () {
  var data;
  var dynamicObj = index_1.observable(new Map());
  index_1.observe(function () {
    data = 0;
    dynamicObj.forEach(function (each) {
      data += each;
    });
  });
  return Promise.resolve().then(function () {
    return expect(data === 0).toBe(true);
  }).then(function () {
    return dynamicObj.set('key0', 3);
  }).then(function () {
    return expect(data === 3).toBe(true);
  }).then(function () {
    return dynamicObj.set('key1', 2);
  }).then(function () {
    return expect(data === 5).toBe(true);
  }).then(function () {
    return dynamicObj.delete('key0');
  }).then(function () {
    return expect(data === 2).toBe(true);
  }).then(function () {
    return dynamicObj.clear();
  }).then(function () {
    return expect(data === 0).toBe(true);
  });
});
test('should not observe non value changing mutations', function () {
  var data;
  var numOfRuns = 0;
  var dynamicObj = index_1.observable(new Map());
  index_1.observe(function () {
    numOfRuns++;
    data = dynamicObj.get('key');
  });
  return Promise.resolve().then(function () {
    expect(data === undefined).toBe(true);
    expect(numOfRuns === 1).toBe(true);
  }).then(function () {
    return dynamicObj.set('key', 'value');
  }).then(function () {
    return dynamicObj.set('key', 'value');
  }).then(function () {
    expect(data === 'value').toBe(true);
    expect(numOfRuns === 2).toBe(true);
  }).then(function () {
    return dynamicObj.delete('key');
  }).then(function () {
    return dynamicObj.delete('key');
  }).then(function () {
    expect(data === undefined).toBe(true);
    expect(numOfRuns === 3).toBe(true);
  });
});
test('should observe map array', function () {
  var data;
  var dynamicObj = index_1.observable(new Map([['a', [1, 2, 3]]]));
  index_1.observe(function () {
    return data = dynamicObj.get('a').length;
  });
  return Promise.resolve().then(function () {
    return expect(data === 3).toBe(true);
  }).then(function () {
    return dynamicObj.get('a').push(4);
  }).then(function () {
    return expect(data === 4).toBe(true);
  });
});
test('should observe map.size', function () {
  var map = index_1.observable(new Map());
  var count = 0;
  index_1.observe(function () {
    count += map.size;
  });
  map.set('banana', 5);
  map.set('apple', 3);
  return Promise.resolve().then(function () {
    return expect(count === 3).toBe(true);
  });
});
/**
 * WeakMap
 */

test('should observe mutations', function () {
  var data;
  var key = {};
  var dynamicObj = index_1.observable(new WeakMap());
  index_1.observe(function () {
    return data = dynamicObj.get(key);
  });
  return Promise.resolve().then(function () {
    return expect(data === undefined).toBe(true);
  }).then(function () {
    return dynamicObj.set(key, 'value');
  }).then(function () {
    return expect(data === 'value').toBe(true);
  }).then(function () {
    return dynamicObj.delete(key);
  }).then(function () {
    return expect(data === undefined).toBe(true);
  });
});
test('should not observe non value changing mutations', function () {
  var data;
  var numOfRuns = 0;
  var key = {};
  var dynamicObj = index_1.observable(new WeakMap());
  index_1.observe(function () {
    numOfRuns++;
    data = dynamicObj.get(key);
  });
  return Promise.resolve().then(function () {
    expect(data === undefined).toBe(true);
    expect(numOfRuns === 1).toBe(true);
  }).then(function () {
    return dynamicObj.set(key, 'value');
  }).then(function () {
    return dynamicObj.set(key, 'value');
  }).then(function () {
    expect(data === 'value').toBe(true);
    expect(numOfRuns === 2).toBe(true);
  }).then(function () {
    return dynamicObj.delete(key);
  }).then(function () {
    return dynamicObj.delete(key);
  }).then(function () {
    expect(data === undefined).toBe(true);
    expect(numOfRuns === 3).toBe(true);
  });
});
/**
 * execution order
 */

test('should run in runner order the first time', function () {
  var data = '';
  var dynamicObj = index_1.observable({
    prop1: 'prop1',
    prop2: 'prop2',
    prop3: 'prop3'
  });
  index_1.observe(function () {
    return data += dynamicObj.prop1;
  });
  index_1.observe(function () {
    return data += dynamicObj.prop2;
  });
  index_1.observe(function () {
    return data += dynamicObj.prop3;
  });
  dynamicObj.prop3 = 'p3';
  dynamicObj.prop1 = 'p1';
  dynamicObj.prop2 = 'p2';
  return Promise.resolve().then(function () {
    return expect(data === 'prop1prop2prop3p3p1p2').toBe(true);
  });
});
/**
 * unobserve
 */

test('should unobserve the observed function', function () {// TODO:
  // let data = ""
  // const dynamicObj = observable({ prop: "" })
  // let numOfRuns = 0
  // function testObserve() {
  //     data = dynamicObj.prop
  //     numOfRuns++
  // }
  // const signal = observe(testObserve)
  // return Promise.resolve()
  //     .then(() => dynamicObj.prop = "Hello")
  //     .then(() => signal.unobserve())
  //     .then(() => {
  //         expect(signal.callback === undefined)
  //         expect(signal.observedKeys === undefined)
  //     })
  //     .then(() => dynamicObj.prop = "World")
  //     .then(() => dynamicObj.prop = "!")
  //     .then(() => expect(numOfRuns === 2))
});
test('should not unobserve if the function is registered for the stack, because of sync', function () {
  var data;
  var dynamicObj = index_1.observable({
    prop: 0
  });
  var numOfRuns = 0;

  function testObserve() {
    data = dynamicObj.prop;
    numOfRuns++;
  }

  var signal = index_1.observe(testObserve);
  return Promise.resolve().then(function () {
    dynamicObj.prop = 2; // but also run

    signal.unobserve();
  }).then(function () {
    return expect(numOfRuns === 2).toBe(true);
  });
});
test('should unobserve even if the function is registered for the stack, when use Action', function () {
  var data;
  var dynamicObj = index_1.observable({
    prop: 0
  });
  var numOfRuns = 0;

  function testObserve() {
    data = dynamicObj.prop;
    numOfRuns++;
  }

  var signal = index_1.observe(testObserve);
  return Promise.resolve().then(function () {
    index_1.Action(function () {
      dynamicObj.prop = 2; // not run

      signal.unobserve();
    });
  }).then(function () {
    return expect(numOfRuns === 1).toBe(true);
  });
});
test('will trace dependency in anywhere', function () {
  var runCount = 0;
  var dynamicObj = index_1.observable({
    a: 0,
    b: 1
  });
  index_1.observe(function () {
    // use a
    // tslint:disable-next-line:no-unused-expression
    dynamicObj.a;
    runSomeThing();
    runCount++;
  });

  function runSomeThing() {
    // use b
    // tslint:disable-next-line:no-unused-expression
    dynamicObj.b;
  }

  dynamicObj.a = 2;
  dynamicObj.b = 3;
  return Promise.resolve().then(function () {
    return expect(runCount === 3).toBe(true);
  });
});
test('Action will not trace dependency', function () {
  var runCount = 0;
  var dynamicObj = index_1.observable({
    a: 0,
    b: 1
  });
  index_1.observe(function () {
    // use a
    // tslint:disable-next-line:no-unused-expression
    dynamicObj.a;
    index_1.Action(function () {
      // tslint:disable-next-line:no-unused-expression
      dynamicObj.b;
    });
    runCount++;
  });
  dynamicObj.b = 2;
  return Promise.resolve().then(function () {
    return expect(runCount === 1).toBe(true);
  });
});
/**
 * Action handle async
 */

test('Action not handle async function!!',
/*#__PURE__*/
(0, _asyncToGenerator2.default)(
/*#__PURE__*/
_regenerator.default.mark(function _callee4() {
  var runCount, num, dynamicObj;
  return _regenerator.default.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          runCount = 0;
          num = 0;
          dynamicObj = index_1.observable({
            a: 0,
            b: 1
          });
          index_1.observe(function () {
            // use a
            num = dynamicObj.a;
            runCount++;
          });
          index_1.Action(
          /*#__PURE__*/
          (0, _asyncToGenerator2.default)(
          /*#__PURE__*/
          _regenerator.default.mark(function _callee3() {
            return _regenerator.default.wrap(function _callee3$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    dynamicObj.a = 1;
                    dynamicObj.a = 2;
                    _context3.next = 4;
                    return Promise.resolve();

                  case 4:
                    dynamicObj.a = 3;
                    dynamicObj.a = 4;
                    dynamicObj.a = 5;
                    dynamicObj.a = 6;

                  case 8:
                  case "end":
                    return _context3.stop();
                }
              }
            }, _callee3, this);
          })));
          return _context4.abrupt("return", Promise.resolve().then(function () {
            return expect(runCount === 2).toBe(false);
          }).then(function () {
            return expect(num === 2).toBe(false);
          }));

        case 6:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4, this);
})));
test('Action handle async function with Action',
/*#__PURE__*/
(0, _asyncToGenerator2.default)(
/*#__PURE__*/
_regenerator.default.mark(function _callee6() {
  var runCount, num, dynamicObj;
  return _regenerator.default.wrap(function _callee6$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          runCount = 0;
          num = 0;
          dynamicObj = index_1.observable({
            a: 0,
            b: 1
          });
          index_1.observe(function () {
            // use a
            num = dynamicObj.a;
            runCount++;
          });
          index_1.Action(
          /*#__PURE__*/
          (0, _asyncToGenerator2.default)(
          /*#__PURE__*/
          _regenerator.default.mark(function _callee5() {
            return _regenerator.default.wrap(function _callee5$(_context5) {
              while (1) {
                switch (_context5.prev = _context5.next) {
                  case 0:
                    dynamicObj.a = 1;
                    _context5.next = 3;
                    return Promise.resolve();

                  case 3:
                    index_1.Action(function () {
                      dynamicObj.a = 2;
                      dynamicObj.a = 3;
                      dynamicObj.a = 4;
                      dynamicObj.a = 5;
                    });

                  case 4:
                  case "end":
                    return _context5.stop();
                }
              }
            }, _callee5, this);
          })));
          return _context6.abrupt("return", Promise.resolve().then(function () {
            return expect(runCount === 3).toBe(true);
          }).then(function () {
            return expect(num === 5).toBe(true);
          }));

        case 6:
        case "end":
          return _context6.stop();
      }
    }
  }, _callee6, this);
})));
/**
 * Branch judgment
 */

test('branch judgment', function () {
  var value = 0;
  var runCount = 0;
  var dynamicObj = index_1.observable({
    a: true,
    b: 1,
    c: 2
  });
  index_1.observe(function () {
    runCount++; // tslint:disable-next-line:prefer-conditional-expression

    if (dynamicObj.a) {
      value = dynamicObj.b;
    } else {
      value = dynamicObj.c;
    }
  });
  return Promise.resolve().then(function () {
    return expect(value === 1).toBe(true);
  }).then(function () {
    return expect(runCount === 1).toBe(true);
  }).then(function () {
    return dynamicObj.c = 3;
  }) // nothing happend
  .then(function () {
    return expect(value === 1).toBe(true);
  }).then(function () {
    return expect(runCount === 1).toBe(true);
  }).then(function () {
    return dynamicObj.a = false;
  }).then(function () {
    return expect(value === 3).toBe(true);
  }).then(function () {
    return expect(runCount === 2).toBe(true);
  }).then(function () {
    return dynamicObj.c = 4;
  }).then(function () {
    return expect(value === 4).toBe(true);
  }).then(function () {
    return expect(runCount === 3).toBe(true);
  }).then(function () {
    return dynamicObj.b = 5;
  }).then(function () {
    return expect(value === 4).toBe(true);
  }).then(function () {
    return expect(runCount === 3).toBe(true);
  });
});
/**
 * Static
 */

test('Static will not tracking object', function () {
  var runCount = 0;
  var result = '';
  var dynamicObj = index_1.observable(index_1.Static({
    name: 'b'
  }));
  index_1.observe(function () {
    result = dynamicObj.name;
    runCount++;
  });
  dynamicObj.name = 'c';
  return Promise.resolve().then(function () {
    return expect(runCount === 1).toBe(true);
  }).then(function () {
    return expect(result === 'b').toBe(true);
  });
});
test('Static will not tracking map', function () {
  var runCount = 0;
  var size = 0;
  var dynamicObj = index_1.observable(index_1.Static(new Map()));
  index_1.observe(function () {
    size = dynamicObj.size;
    runCount++;
  });
  dynamicObj.set('a', 'a');
  dynamicObj.set('b', 'b');
  return Promise.resolve().then(function () {
    return expect(runCount === 1).toBe(true);
  }).then(function () {
    return expect(size === 0).toBe(true);
  });
});
test('nested observe should eventually run', function () {
  var dynamicObj = index_1.observable({
    a: '1',
    b: '2',
    c: '3',
    d: '4'
  });
  var str = '';
  index_1.observe(function () {
    str += dynamicObj.a;
    index_1.observe(function () {
      str += 5;
      index_1.observe(function () {
        str += dynamicObj.b;
      });
      str += 6;
    });
    index_1.observe(function () {
      str += 8;
      index_1.observe(function () {
        str += dynamicObj.c;
      });
      str += 9;
    });
    str += dynamicObj.d;
  });
  dynamicObj.a = '2';
  return Promise.resolve().then(function () {
    return expect(str === '1456289324562893').toBe(true);
  });
});
test('observe in action', function () {
  var dynamicObj = index_1.observable({
    a: 'a'
  });
  var count = '';
  index_1.Action(function () {
    index_1.observe(function () {
      count += dynamicObj.a;
    });
    count += 'b';
  });
  dynamicObj.a = 'c';
  return Promise.resolve().then(function () {
    return expect(count === 'bac').toBe(true);
  });
});